package com.abmcode.bpmsample.screen.form.leaveapplication;

import com.abmcode.bpmsample.entity.LeaveAppStatus;
import com.abmcode.bpmsample.entity.User;
import io.jmix.bpmui.processform.ProcessFormContext;
import io.jmix.bpmui.processform.annotation.Outcome;
import io.jmix.bpmui.processform.annotation.OutputVariable;
import io.jmix.bpmui.processform.annotation.ProcessForm;
import io.jmix.bpmui.processform.annotation.ProcessVariable;
import io.jmix.core.DataManager;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.TextArea;
import io.jmix.ui.screen.*;
import com.abmcode.bpmsample.entity.LeaveApplication;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.UUID;

@UiController("bpmspl_LeaveApplication.form")
@UiDescriptor("leave-application-form.xml")
@EditedEntityContainer("leaveApplicationDc")
@ProcessForm(
        outcomes = {
//                @Outcome(id = "new",
//                        outputVariables = {
//                                @OutputVariable(name = "leaveApplication", type = LeaveApplication.class)
//                        }
//                ),
                @Outcome(id = "approve"),
                @Outcome(id = "reject")
        }
)
public class LeaveApplicationForm extends StandardEditor<LeaveApplication> {

    @Autowired
    private Button submitBtn;
    @Autowired
    private Button approveBtn;
    @Autowired
    private Button rejectBtn;
    @Autowired
    private ProcessFormContext processFormContext;

    @ProcessVariable
    private LeaveApplication leaveApplication;

    @Autowired
    @ProcessVariable(name = "comments")
    private TextArea<String> commentsField;

    @ProcessVariable
    private UUID leaveAppId;

    @Autowired
    private CurrentAuthentication currentAuthentication;

    private boolean isSubmit = false;
    @Autowired
    private DataManager dataManager;

    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        if (leaveApplication == null) {
            leaveApplication = getEditedEntity();
        }

        // 新请假
        if (leaveApplication.getEmployee() == null) {
            leaveApplication.setStatus(LeaveAppStatus.NEW);
            User user = (User) currentAuthentication.getUser();
            leaveApplication.setEmployee(user);
            isSubmit = true;
        }

        leaveAppId = leaveApplication.getId();

        submitBtn.setEnabled(isSubmit);
        approveBtn.setEnabled(!isSubmit);
        rejectBtn.setEnabled(!isSubmit);
    }


    @Subscribe("submitBtn")
    public void onSubmitClick(Button.ClickEvent event) {
        leaveApplication = dataManager.save(leaveApplication);

        processFormContext.processStarting()
                .addProcessVariable("NPE",true)
                .saveInjectedProcessVariables()
                .start();
        closeWithDiscard();
    }

    @Subscribe("approveBtn")
    public void onApproveClick(Button.ClickEvent event) {
        User user = (User) currentAuthentication.getUser();
        leaveApplication.setApproval(user);

        processFormContext.taskCompletion()
                .withOutcome("approve")
                .saveInjectedProcessVariables()
                .complete();
        closeWithDiscard();
    }

    @Subscribe("rejectBtn")
    public void onRejectClick(Button.ClickEvent event) {
        User user = (User) currentAuthentication.getUser();
        leaveApplication.setApproval(user);

        processFormContext.taskCompletion()
                .withOutcome("reject")
                .addProcessVariable("comment", commentsField.getValue())
                .complete();
        closeWithDiscard();
    }
}