package com.abmcode.bpmsample.screen.leaveapplication;

import io.jmix.bpmui.processform.ProcessFormContext;
import io.jmix.bpmui.processform.ProcessFormContextObjectFactory;
import io.jmix.bpmui.processform.ProcessFormScreens;
import io.jmix.bpmui.processform.annotation.ProcessVariable;
import io.jmix.core.DataManager;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.Table;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import com.abmcode.bpmsample.entity.LeaveApplication;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.repository.ProcessDefinition;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Set;

@UiController("bpmspl_LeaveApplication.browse")
@UiDescriptor("leave-application-browse.xml")
@LookupComponent("leaveApplicationsTable")
public class LeaveApplicationBrowse extends StandardLookup<LeaveApplication> {

    private static final String PROCESS_KEY = "leave-application";
    @Autowired
    protected DataManager dataManager;

    @ProcessVariable
    private LeaveApplication leaveApplication;

    @Autowired
    private RepositoryService repositoryService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private TaskService taskService;

    @Autowired
    protected ProcessFormScreens processFormScreens;

    @Autowired
    private Notifications notifications;

    @Autowired
    private Table<LeaveApplication> leaveApplicationsTable;
    @Autowired
    private CollectionLoader<LeaveApplication> leaveApplicationsDl;

    @Subscribe("createBtn")
    public void onCreateBtnClick(Button.ClickEvent event) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery()
                .processDefinitionKey(PROCESS_KEY)
                .latestVersion()
                .singleResult();

        if (processDefinition == null) {
            notifications.create()
                    .withCaption("无此流程")
                    .withDescription("流程：[" + PROCESS_KEY + "] 不存在")
                    .withType(Notifications.NotificationType.ERROR)
                    .show();
            return;
        }

        Screen startProcessForm = processFormScreens.createStartProcessForm(processDefinition, this);

        leaveApplication = dataManager.create(LeaveApplication.class);

        if (startProcessForm instanceof EditorScreen) {
            ((EditorScreen<LeaveApplication>) startProcessForm).setEntityToEdit(leaveApplication);
        }

        startProcessForm.addAfterCloseListener(afterCloseEvent -> {
            refreshTable();
        });

        startProcessForm.show();
    }

    @Subscribe("approveBtn")
    public void onApproveClick(Button.ClickEvent event) {
        Set<LeaveApplication> leaveApps = leaveApplicationsTable.getSelected();

        if (leaveApps.isEmpty()) {
            notifications.create()
                    .withType(Notifications.NotificationType.HUMANIZED)
                    .withDescription("请选择要审批的请假")
                    .show();
            return;
        }

        // 拿到表格中的请假实例，并设置为流程变量
        leaveApplication = leaveApps.stream().findFirst().get();

        // 根据实例的 ID 查询流程变量。这里要在前一步启动流程时，将实例的 id 存在流程变量 "leaveAppId" 里。
        List<ProcessInstance> procInstances = runtimeService.createProcessInstanceQuery()
                .processDefinitionKey(PROCESS_KEY)
                .variableValueEquals("leaveAppId", leaveApplication.getId())
                .active()
                .list();

        ProcessInstance procInstance = procInstances.get(0);

        // 根据流程实例，查找对应的任务
        List<Task> tasks = taskService.createTaskQuery()
                .processInstanceId(procInstances.get(0).getId())
                .active()
                .list();

        // 用任务创建流程表单
        Screen taskForm = processFormScreens.createTaskProcessForm(tasks.get(0), this);

        // 设置表单中的实体
        if (taskForm instanceof EditorScreen) {
            ((EditorScreen<LeaveApplication>) taskForm).setEntityToEdit(leaveApplication);
        }

        // 审批结束后刷新表格
        taskForm.addAfterCloseListener(afterCloseEvent -> {
            refreshTable();
        });

        taskForm.show();
    }

    public void refreshTable(){
        leaveApplicationsDl.load();
    }
}